//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//file oriperm.h
//
//In the class OPerm, the orientations are "attached" to the permutations.
//If o is an instance of OPerm, and o.perm[i]=x and o.ori[i]=y this means, that the
//element i is mapped to the element x and the orientation is incremented
//by y (modulo order). This is exactly what happens when applying moves to
//a cube or turning the whole cube. Note: Reflections of cube corners cannot
//be handled with this concept, so we must be careful here.
//
//The classes COPerm and EOPerm are derived from OPerm for the case of edges
//and corners. See COPerm operator*(COPerm&) how reflections at the RL-plane are
//handled.
//
//COPerm COPerm::CubeSym(int x): defines a cube symmetry for 0<=x<48
//
//char COPerm::CubeSymIdx(COPerm&): is defined by CubeSymIdx(CubeSym(x))=x
//
//void COPerm::InitSymmetryGroupTables(): creates the tables symMult and symInv
//for the multiplication and the inverse of cube symmetries.
//If we have two symmetries S1 and S2 with indices i and j, symMult[i][j]
//gives the index of S1*S2. symInv[i] holds the index of the inverse symmetry S1^-1
//of S1.
//
//void InitMoveConjugateTable(): A move T is transformed into a move R by
//the conjugation S * T * S^-1, where S is a cube symmetry.
//moveConj[s][t]=r stores this result, where s, r and t are the corresponding
//indices.
//
//void EOPermutation::MakeFlipSliceRepTable()://the table maps for each class
//the corresponding representants "raw" flipSlice-coordinate.
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#ifndef ORIPERM_H
#define ORIPERM_H

#include "permutation.h"
#include "orientation.h"

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class OPerm
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
{  public:
	OPerm(char size, char order);
//	OPerm(int a,int b);
	OPerm(const Perm& p,char order);
	OPerm(const Ori& o);
	OPerm(const Perm&p, const Ori& o);
	OPerm(const OPerm& op);//copy constructor
	OPerm(char size, char order, int pIdx, int oIdx);

	OPerm& operator=(OPerm&);
	OPerm operator*(OPerm&);
	OPerm Inverse();
	void Print();
	int PIndex();
	int OIndex();

	Perm perm;
	Ori ori;
};

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class COPerm: public OPerm
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
{	public:
	COPerm();
	COPerm(const CPerm&);
	COPerm(int pIndex, int oIndex);
	COPerm(const COri&);
	COPerm(const CPerm&, const COri&);
	COPerm(const COPerm& op);
	COPerm(const OPerm& op);

	COPerm Turn(Move T);

	COPerm operator*(COPerm&);
	COPerm Inverse();

	static COPerm CubeSym(int x);
	static char CubeSymIdx(const COPerm& );

	static void InitSymmetryGroupTables();
	static void InitMoveConjugateTable();

	unsigned short Pos();
	unsigned short Twist();

	static COPerm U,R,F,D,L,B,M_U4,M_URF3,M_F2,M_LR2;
};

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class EOPerm: public OPerm
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
{	public:
	EOPerm();
	EOPerm(const EPerm&);
	EOPerm(const EOri&);
	EOPerm(const EPerm&, const EOri&);
	EOPerm(const EOPerm& eop);
	EOPerm(const OPerm& op);
	EOPerm(int pIndex, int oIndex);
	EOPerm(int flipSlice);
	EOPerm(short int sortedSlice);
	EOPerm(unsigned char* generatorAxis, unsigned char* generatorPower);

	static EOPerm U,R,F,D,L,B,M_U4,M_URF3,M_F2,M_LR2;

	EOPerm operator*(EOPerm&);
	EOPerm Inverse();
	
	void Turn(Move T);
	unsigned short Flip();
	unsigned short Slice();
	unsigned short SortedSlice();
	int FlipSlice();

	static void InitCubeSymTable();
	static void MakeFlipSliceRepTable();
	static EOPerm CubeSym(int x);
};

#endif